// ModelSwarm.java
// The top-level ModelSwarm.

import swarm.Globals;
import swarm.Selector;
import swarm.defobj.Zone;
import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;
import swarm.activity.ActionGroupImpl;
import swarm.activity.ScheduleImpl;
import swarm.activity.Activity;



public class ModelSwarm extends SwarmImpl {
    
    // Declare the model parameters and their default values.

    public int worldXSize = 80, worldYSize = 80;
    public double seedProb = 0.20;
    public int endTime = 200000;
    
    FoodSpace foodSpace;
    SimpleBug abug;
    ScheduleImpl modelSchedule;

    public ModelSwarm(Zone aZone) {
        super(aZone);
    }

    public Object buildObjects(){
    
        int xPos, yPos;

        super.buildObjects();
        
        // Now create the model's objects.
        // First create the foodspace and seed it with food.
        
        foodSpace = new FoodSpace(Globals.env.globalZone,
        worldXSize, worldYSize);
        foodSpace.seedFoodWithProb(seedProb);
       
        // Find the middle of the foodspace.
        
        xPos = (foodSpace.getSizeX())/2;
        yPos = (foodSpace.getSizeY())/2;
        
        // Create a single SimpleBug, abug, and place it in the
        // foodspace at (xPos, yPos). The bug knows the size of its
        // world from being passed the pointer to the foodspace in
        // which it is to be created.
        
        abug = new SimpleBug(Globals.env.globalZone, foodSpace,
        xPos, yPos, 1);
        return this;
}

    // This is the method a) for building the list of actions for
    // these objects to accomplish and b) for scheduling these actions
    // in simulated time.
    
    public Object buildActions() {
    
        ActionGroupImpl modelActions;

        // First, use the parent class to initialize the process.
        
        super.buildActions();
        modelActions = new ActionGroupImpl(getZone());
        
        try {
            modelActions.createActionTo$message
	       (abug,  new Selector(abug.getClass(), "randomWalk", false));
        } 
        
        catch (Exception e) {
            System.err.println("Exception randomWalk: " +
            e.getMessage ());
            System.exit(1);
        }

        // Next we add to modelActions another message, this one to
        // modelSwarm itself telling it to check the simulation
        // time. We use this to send a timestamp to the console and to
        // stop the model after a specified number of periods.
        
        try {
            modelActions.createActionTo$message
		(this, new Selector(this.getClass(), "checkTime", false));
        } 
        
        catch (Exception e) {
            System.err.println("Exception checkTime " + e.getMessage ());
            System.exit(1);
        }
        
        // Now create the schedule and set the repeat interval to unity.
        
        modelSchedule = new ScheduleImpl(getZone(), 1);
       
        // Finally, insert the action list into the schedule at period zero
        
        modelSchedule.at$createAction(0, modelActions);
        
        return this;
}
// This method specifies the context in which the model is to be run.
public Activity activateIn(Swarm swarmContext)
{
// Use the parent class to activate ourselves in the context
// passed to us.
super.activateIn(swarmContext);
// Then activate the schedule in ourselves.
modelSchedule.activateIn(this);
// Finally, return the activity we have built.
return getActivity();
}

// This is a pretty crude method to end the simulation after an
// arbitrary number of periods given by the endTime parameter. If
// the simulation time returned by getCurrentTime() is greater
// than endTime, we send a message to the console and terminate
// the current activity. We also use this method to send a
// timestamp to the console.

/* Verandering: de tijd moet ook worden afgedrukt als een bug voedsel vindt.
   Dat kan niet met deze methode omdat SimpleBug deze methode in ModelSwarm
   niet kan aanroepen. In principe kan dat wel omdat deze methode public is, maar 
   dan zou SimpleBug een pointer naar ModelSwarm moeten hebben en dat zou weer
   allerlei ad hoc trucs met zich meebrengen. Ook even uitproberen of deze public
   methode in private veranderd kan worden.
   
   NEE!! Dat kan dus niet. Door dat messenging systeem van Objective C - de methode
   wordt door een ander object aangeroepen dan door ModelSwarm zelf - kunnen dit
   soort simpele object-oriented regels niet doorgevoerd worden!! 
   Error gegenereerd: Exception checkTime null 
   nog een nixzeggende foutmelding ook! */


    public void checkTime(){
        int i, t;
        int interval = 5;
        if (Globals.env.getCurrentTime() >= endTime) {
        
            // Terminate the simulation.
            System.out.println("We've reached our endTime at period " +
            Globals.env.getCurrentTime());
            Globals.env.getCurrentSwarmActivity().terminate();
        }
        else {
            for (i = 0; (t = (int)Math.pow(interval, (double)i)) <= endTime; i++)
            if (t == Globals.env.getCurrentTime())
                System.out.println("The time is " + t);
       }
       return;
    }
}
